@@ -1,15 +1,17 @@
 predict.rfsrc <-
   function(object,
            newdata,
-           m.target=NULL,
-           importance = c(FALSE, TRUE, "none", "permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble")[1],
-           err.block = NULL,
+           m.target = NULL,
+           importance = c(FALSE, TRUE, "none", "permute", "random", "anti"),
+           block.size = NULL,
+           ensemble = NULL,
            na.action = c("na.omit", "na.impute"),
            outcome = c("train", "test"),
            proximity = FALSE,
            forest.wt = FALSE,
            ptn.count = 0,
             
+           distance = FALSE,
            var.used = c(FALSE, "all.trees", "by.tree"),
            split.depth = c(FALSE, "all.trees", "by.tree"),
            seed = NULL,
@@ -20,21 +22,23 @@ predict.rfsrc <-
 {
   result.predict <- generic.predict.rfsrc(object,
                                           newdata,
+                                          ensemble = ensemble,
                                           m.target = m.target,
                                           importance = importance,
-                                          err.block = err.block,
+                                          block.size = block.size,
                                           na.action = na.action,
                                           outcome = outcome,
                                           proximity = proximity,
                                           forest.wt = forest.wt,
                                           ptn.count = ptn.count,
-                                           
+                                          distance = distance,                                          
                                           var.used = var.used,
                                           split.depth = split.depth,
                                           seed = seed,
                                           do.trace = do.trace,
                                           membership = membership,
                                           statistics = statistics,
+                                           
                                           ...)
   return(result.predict)
 }
